/// @file		registry_access_demo.c
/// @copyright  Copyright (c) 2014 Toradex AG.
///				[Software License Agreement]
/// $Author:	khv $
/// $Date: 		2014-06-17 $
/// @brief:		Registry access demo in VC++.

/// Header files.
#include <windows.h>

int wmain()
{
	DWORD dataType = 0;
	DWORD retVal = 0;
	HKEY retKey;
	HKEY retNewKey;
	DWORD dispositionValue = 0;
	WCHAR dataWriteBuffer[] = L"Welcome to Toradex.";
	DWORD writeBufferSize = sizeof(dataWriteBuffer);
	BYTE *receiveBuffer = (BYTE*)calloc(50, sizeof(BYTE));
	DWORD readBufferSize = 50;

	printf("***** Demo to show registry access in VC++ ***** \n");
		
	/// Opens the key.
	retVal = RegOpenKeyEx(HKEY_CURRENT_USER, TEXT("ControlPanel"), 0, KEY_ALL_ACCESS, &retKey);
	if (retVal != ERROR_SUCCESS)
	{
		printf("\nRegistry opening procedure failed with error code = %u", retVal);
		printf("\nPress enter to exit.");
		fflush(stdin);
		getchar();
		return FALSE;
	}
	printf("\nKey \"HKEY_CURRENT_USER\\ControlPanel\" opened.");

	/// Creates new key.
	retVal = RegCreateKeyEx(retKey, TEXT("My Custom Key"), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &retNewKey, &dispositionValue);
	if (retVal != ERROR_SUCCESS)
	{
		printf("\nKey creating procedure failed with error code = %u", retVal);
		printf("\nPress enter to exit.");
		fflush(stdin);
		getchar();
		return FALSE;
	}	
	printf("\nNew key created with name \"My Custom Key\".");

	/// Creates new value if doesn't exist before, otherwise modifies the existing value.
	retVal = RegSetValueEx(retNewKey, TEXT("My Settings"), 0, REG_SZ, (BYTE*)dataWriteBuffer, writeBufferSize);
	if (retVal != ERROR_SUCCESS)
	{
		printf("\nValue creating procedure failed with error code = %u", retVal);
		printf("\nPress enter to exit.");
		fflush(stdin);
		getchar();
		return FALSE;
	}	
	printf("\nNew value created with name \"My Settings\" with data = %ls", dataWriteBuffer);

	/// Reads the value.
	retVal = RegQueryValueEx(retNewKey, TEXT("My Settings"), NULL, &dataType, receiveBuffer, &readBufferSize);
	if (retVal != ERROR_SUCCESS)
	{
		printf("\nValue reading procedure failed with error code = %u", retVal);
		printf("\nPress enter to exit.");
		fflush(stdin);
		getchar();
		return FALSE;
	}	
	printf("\nData read from \"My Settings\" value = %ls", receiveBuffer);
	
	printf("\nPress enter to delete \"My Settings\" value.");
	fflush(stdin);
	getchar();

	/// Deletes the value.
	retVal = RegDeleteValue(retNewKey, TEXT("My Settings"));
	if (retVal != ERROR_SUCCESS)
	{
		printf("Value deleting procedure failed with error code = %u", retVal);
		printf("\nPress enter to exit.");
		fflush(stdin);
		getchar();
		return FALSE;
	}	
	printf("Value deleted.");

	printf("\nPress enter to delete \"My Custom Key\" key.");
	fflush(stdin);
	getchar();

	/// Note - Close any open handle before deleting that key, otherwise delete key procedure will fail.
	/// Like in this case handle "retNewKey" is open to "My Custom Key" key.
	/// So firstly close handle "retNewKey", and then delete "My Custom Key" key.
	RegCloseKey(retNewKey);				

	/// Deletes the key.
	retVal = RegDeleteKey(retKey, TEXT("My Custom Key"));
	if (retVal != ERROR_SUCCESS)
	{
		printf("Key deleting procedure failed with error code = %u", retVal);
		printf("\nPress enter to exit.");
		fflush(stdin);
		getchar();
		return FALSE;
	}	
	printf("key deleted.");

	/// Saves the registry.
	retVal = RegFlushKey(retKey);
	if (retVal != ERROR_SUCCESS)
	{
		printf("Registry saving procedure failed with error code = %u", retVal);
	}
	else
	{
		printf("\nKey \"HKEY_CURRENT_USER\\ControlPanel\" saved successfully.");
	}

	/// Closes the open key.
	RegCloseKey(retKey);
	printf("\nPress enter to exit.");
	fflush(stdin);
	getchar();
	return TRUE;
}